#!/usr/bin/env python

#Copyright (c) 2015 Ben Letham
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
#The author/copyright holder can be contacted at bletham@gmail.com
from collections import defaultdict
from math import log

#FILL IN DATA HERE
data_filename = '1978_snowfall.txt' #the file containing daily snowfall amounts for the winter
data_unit = 'mm' #the unit of the snowfall amounts. Must be either 'mm' or 'in'.



#These are the functions used:

#Load in the snowfall data. Output is a list of daily snowfall amounts, in inches.
def get_data(fname, unit):
    with open(fname, 'rb') as fin:
        snowfall_data = fin.readlines()
    if unit == 'mm':
        unit_fctr = 10.*2.54 #convert mm to inches
    elif unit == 'in':
        unit_fctr = 1.
    #Convert the snow data to float inches
    snowfall_data = [float(datum)/unit_fctr for datum in snowfall_data]
    return snowfall_data

#Compute the period accumulation curve. The output is a list of the period accumulation y-value for n=0,149 days
def period_accumulation_curve(snowfall_data):
    period_accum = defaultdict(int)
    dts = [i for i,d in enumerate(snowfall_data) if d>0] #these are the indicies of days with non-zero snowfall
    for i,dt in enumerate(dts):
        #For each day with non-zero snowfall, we will compute the snowfall in all time periods that start at this date.
        #First, we see if this day beats the 1-day snowfall record
        period_accum[0] = max(period_accum[0],snowfall_data[dt])
        #And now we do the same comparison for all other intervals starting at this date
        j = 1
        intrvlsum = float(snowfall_data[dt]) #the cumulative snowfall since dt
        while i+j < len(dts):
            intrvlsum += snowfall_data[dts[i+j]] #we've increased our time period to include the next day with non-zero snowfall
            period_accum[dts[i+j] - dts[i]] = max(period_accum[dts[i+j] - dts[i]],intrvlsum) #Was this the most snow we've seen in an interval of this size?
            j+=1
    #Now fill in any missing days, and replace values with the max of its subintervals, if higher.
    for i in range(1,149):
        #We are interested in range(149) as this is the interval over which the L-score is calculated, after pre-pending 0.
        period_accum[i] = max(period_accum[i],period_accum[i-1])
    #And now switch to a list, starting at 0.
    period_accum_vals = [0]
    period_accum_vals.extend([period_accum[i] for i in range(149)])
    return period_accum_vals

#Compute the L-score from the period accumulation score.
def compute_Lscore(period_accum):
    x = range(150) #the x-axis in days
    logx = [log(1+x_i) for x_i in x] #the x-axis moved to a log scale
    Lscore = trapezoid_intgrl(logx,period_accum)/29.73 #compute the integral, and normalize to the score value
    Lscore_rounded = round(Lscore,1) #Round to 1 decimal point
    return Lscore, Lscore_rounded

#The trapezoid rule for approximating an integral
def trapezoid_intgrl(x,f):
    y = 0.
    for i in range(len(x)-1):
        y += (x[i+1]-x[i])*(f[i+1]+f[i])
    y *=0.5
    return y

#Print (and interpret) the L-score
def print_Lscore(Lscore, Lscore_rounded):
    guide = [(2., 'easy'), (4., 'snowy'), (6., 'hard'), (8., 'severe'), (10., 'brutal'), (1e9, 'time to move')]
    for limit,interp in guide:
        if Lscore <= limit:
            Lscore_interp = str(interp)
            break
    print "This winter's L-score was "+str(Lscore_rounded)+" ("+Lscore_interp+")."
    raw_input('Press Enter to quit.')
    raise SystemExit(0)


if __name__ == '__main__':
    snowfall_data = get_data(data_filename, data_unit) #load in the snowfall data
    period_accum = period_accumulation_curve(snowfall_data) #Compute the period accumulation curve
    Lscore, Lscore_rounded = compute_Lscore(period_accum) #Compute the period accumulation curve
    print_Lscore(Lscore, Lscore_rounded) #